// UWindoid.h -----------------------------------------------------------------
// Copyright  1991 by Apple Computer, Inc. All rights reserved.

#pragma once


// INCLUDE FILES --------------------------------------------------------------

#include "IncludeFiles.h"


//FORWARD DECLARATIONS --------------------------------------------------------

class TFloatingMenu;
class TDefaultView;
class TLightPalette;


// GLOBALS --------------------------------------------------------------------

TFloatingMenu* 	gFloatingMenu;
TWindow*		gFloatMenuWindow;

enum eLights {kGreen=0,kYellow,kRed, kNoTool = 42}; // global values for the lights	


// CLASS DEFINITIONS-----------------------------------------------------------

// TApplication ---------------------------------------------------------------

class TWindoidApplication : public TApplication {
public:
//	METHODS
	virtual pascal void 		IWindoidApplication(OSType itsMainFileType,
													OSType itsCreator);
	virtual pascal void			DoSetupMenus();
	virtual pascal TDocument* 	DoMakeDocument(CommandNumber, TFile* itsFile);
};


// TDocuments -----------------------------------------------------------------

class TWindoidDocument : public TDocument {
public:
//	METHODS
	virtual pascal void DoMakeViews(Boolean forPrinting); 

	virtual pascal void	DoSetupMenus();				// setup document menus
	virtual pascal void DoMenuCommand(CommandNumber aCommandNumber);
	virtual pascal void DoLight(eLights);			// do Light drawing

//	FIELDS	
	TDefaultView*		fView;						// the view used 	
};


// TViews ---------------------------------------------------------------------

class TDefaultView : public TView {
public:
//	ENUMS AND TYPEDEFS
	enum eCoordinate {ktop=20,kbottom=150,kleft=200,kright=250};

//	METHODS
	virtual pascal void		Initialize();
	
	virtual pascal void 	Draw(const VRect& area);// general draw method
	virtual pascal void 	DrawTrafficLight();		// draw traffic light body
	virtual pascal void 	DrawLight();			// general light drawing method
	virtual pascal void 	DrawGreen();			// draw green light
	virtual pascal void 	DrawYellow();			// draw yellow light
	virtual pascal void 	DrawRed();				// draw red light

//	FIELDS	
	Rect 	fTrafficRect;							// traffic light components
	Rect 	fBlackTopRect;
	Rect 	fTrafficLightFrame;
	Rect 	fGreenRect;
	Rect 	fYellowRect;
	Rect 	fRedRect;
	Str255 	fTitle;
	eLights	fCurrentLight;
};


// And now the cool stuff, the TearOff floating window class

class TFloatingMenu : public TTearOffMenuView {
public:
//	METHODS
	virtual pascal void IFloatingMenu();		// init floating menu
};


class TLightPalette : public TView {
public:
//	METHODS
	virtual pascal void ILightPalette(TDocument *itsDocument, TView *itsView);
	
	virtual pascal void Draw(const VRect &area);	// draw view
	virtual pascal void Activate(Boolean entering);	// activate view
	virtual pascal void IRes(TDocument* itsDocument, TView* itsSuperView,
							TStream* itsParams);
	
	virtual pascal void DoMouseCommand	   (VPoint& theMouse, TToolboxEvent* event,
											Point hysteresis);
											
	virtual pascal void TrackFeedback	   (TrackPhase 		aTrackPhase,
											const VPoint&	anchorPoint,
											const VPoint&	previousPoint,
											const VPoint&	nextPoint,
											Boolean			mouseDidMove,
											Boolean			turnItOn);

	
	virtual pascal void SelectNewTool(eLights whichTool);
	virtual pascal void FrameTool(Rect&);			// frame the selected tool
	virtual pascal void BuildChoiceArray();			// build tool array
	virtual pascal void Toggle();					// toggle tool selections
	virtual pascal void DoHighlightSelection(HLState fromHL, HLState toHL);

//	FIELDS	
	Rect 	fChoiceArray[kLightsInPalette];			// tool array
	eLights	fCurrentTool;							// tool selection
	eLights	fOldTool;								// earlier used tool
	eLights	fSelectedTool;							// currently *selected* tool						
};


// TCOMMANDS ------------------------------------------------------------------

class TToolSelectCommand : public TTearOffMenuViewTracker {
public:
//	METHODS
	virtual pascal void 	IToolSelectCommand (TLightPalette* 	theToolsPalette,
										 		TLightPalette*	theMenuToolsPalette,
												TLightPalette* 	theFloatingToolsPalette,
												VPoint& 		theMouse);
											
	virtual pascal  void 	DoIt(void);				// execute selection command
	virtual Boolean pascal	IsDoneTracking();		// no more tracking?

	virtual pascal	void	TrackFeedback	   (TrackPhase 		aTrackPhase,
												const VPoint&	anchorPoint,
												const VPoint&	previousPoint,
												const VPoint&	nextPoint,
												Boolean			mouseDidMove,
												Boolean			turnItOn);
												
	virtual pascal TTracker* TrackMouse		   (TrackPhase 		aTrackPhase,
												VPoint&			anchorPoint,
												VPoint&			previousPoint,
												VPoint&			nextPoint,
												Boolean			mouseDidMove);
												
//	FIELDS												
	eLights			fTool;							// selected tool
	TLightPalette*	fMenuToolsPalette;				// ptr to Menu Tool view
	TLightPalette*	fFloatingToolsPalette;			// ptr to Floating WIndow view
	Boolean			fExitTracking;					// keep track of tracking state
};


class TTrafficCommand : public TCommand {
public:
//	METHODS
	TWindoidDocument* fCommandDocument;				// document attached to command
};


class TGreenCommand : public TTrafficCommand {
public:
//	METHODS
	virtual pascal void IGreenCommand(TWindoidDocument*);
	virtual pascal void DoIt();
};


class TYellowCommand : public TTrafficCommand {
public:
//	METHODS
	virtual pascal void IYellowCommand(TWindoidDocument*);
	virtual pascal void DoIt();
};


class TRedCommand : public TTrafficCommand {
public:
//	METHODS
	virtual pascal void IRedCommand(TWindoidDocument*);
	virtual pascal void DoIt();
};


// THE END -------------------------------------------------------------------
